from .geom import *

class Polygon:
    class Vertex:
        def __init__(self, polygon, point, index):
            self.polygon = polygon
            self.point = point
            self.index = index

        def __repr__(self):
            return "<({0}, {1})>".format(*self.point)

        @property
        def x(self):
            return self.point[0]

        @property
        def y(self):
            return self.point[1]

        def next(self):
            return self.polygon[(self.index+1)%len(self.polygon)]

        def previous(self):
            return self.polygon[(self.index+1+len(self.polygon))%len(self.polygon)]

        def __getitem__(self, n):
            return self.point[n]

    def __init__(self, vertices):
        self.vertices = list()

        self.bottomLeft = (99e99, 99e99)
        self.topRight = (-99e999, -99e99)

        for point in vertices:
            if point[0] < self.bottomLeft[0]:
                self.bottomLeft = (point[0], self.bottomLeft[1])
            if point[1] < self.bottomLeft[1]:
                self.bottomLeft = (self.bottomLeft[0], point[1])

            if point[0] > self.topRight[0]:
                self.topRight = (point[0], self.topRight[1])
            if point[1] > self.topRight[1]:
                self.topRight = (self.topRight[0], point[1])

            vertex = Polygon.Vertex(self, point, len(self.vertices))
            self.vertices.append(vertex)

    def __getitem__(self, item):
        return self.vertices[item]

    def __len__(self):
        return len(self.vertices)

    def __iter__(self):
        return iter(self.vertices)

    def point_in_polygon(self, point):
        #Test bounding box
        if self.bottomLeft[0] < point[0]+EPSILON and \
                point[0]-EPSILON < self.topRight[0] and \
                self.bottomLeft[1] < point[1]+EPSILON and \
                point[1]-EPSILON < self.topRight[1]:
            #Inside bounding box

            origin = self.bottomLeft

            if sign(point[0] - origin[0])==0 and sign(point[1] - origin[1]):
                #Point is the origin. Outside or in the border.
                for v in self.vertices:
                    if point_in_segment(point, v.point, v.next().point):
                        return 1
                return 0

            sum_triangles = 0

            for vertex in self.vertices:
                current = (vertex.point, vertex.next().point)
                if point_in_segment(point, *current):
                    return 1

                triangle = (origin, *current)
                sum_triangles += point_in_triangle(point, *triangle)

            return sum_triangles
        return 0
